from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

SDL_SET_CURRENT_BINARY(SDL_BINARY)

SDL_Scancode = ctypes.c_uint32

SDL_SCANCODE_UNKNOWN = 0

SDL_SCANCODE_A = 4
SDL_SCANCODE_B = 5
SDL_SCANCODE_C = 6
SDL_SCANCODE_D = 7
SDL_SCANCODE_E = 8
SDL_SCANCODE_F = 9
SDL_SCANCODE_G = 10
SDL_SCANCODE_H = 11
SDL_SCANCODE_I = 12
SDL_SCANCODE_J = 13
SDL_SCANCODE_K = 14
SDL_SCANCODE_L = 15
SDL_SCANCODE_M = 16
SDL_SCANCODE_N = 17
SDL_SCANCODE_O = 18
SDL_SCANCODE_P = 19
SDL_SCANCODE_Q = 20
SDL_SCANCODE_R = 21
SDL_SCANCODE_S = 22
SDL_SCANCODE_T = 23
SDL_SCANCODE_U = 24
SDL_SCANCODE_V = 25
SDL_SCANCODE_W = 26
SDL_SCANCODE_X = 27
SDL_SCANCODE_Y = 28
SDL_SCANCODE_Z = 29

SDL_SCANCODE_1 = 30
SDL_SCANCODE_2 = 31
SDL_SCANCODE_3 = 32
SDL_SCANCODE_4 = 33
SDL_SCANCODE_5 = 34
SDL_SCANCODE_6 = 35
SDL_SCANCODE_7 = 36
SDL_SCANCODE_8 = 37
SDL_SCANCODE_9 = 38
SDL_SCANCODE_0 = 39

SDL_SCANCODE_RETURN = 40
SDL_SCANCODE_ESCAPE = 41
SDL_SCANCODE_BACKSPACE = 42
SDL_SCANCODE_TAB = 43
SDL_SCANCODE_SPACE = 44

SDL_SCANCODE_MINUS = 45
SDL_SCANCODE_EQUALS = 46
SDL_SCANCODE_LEFTBRACKET = 47
SDL_SCANCODE_RIGHTBRACKET = 48
SDL_SCANCODE_BACKSLASH = 49
SDL_SCANCODE_NONUSHASH = 50
SDL_SCANCODE_SEMICOLON = 51
SDL_SCANCODE_APOSTROPHE = 52
SDL_SCANCODE_GRAVE = 53
SDL_SCANCODE_COMMA = 54
SDL_SCANCODE_PERIOD = 55
SDL_SCANCODE_SLASH = 56

SDL_SCANCODE_CAPSLOCK = 57

SDL_SCANCODE_F1 = 58
SDL_SCANCODE_F2 = 59
SDL_SCANCODE_F3 = 60
SDL_SCANCODE_F4 = 61
SDL_SCANCODE_F5 = 62
SDL_SCANCODE_F6 = 63
SDL_SCANCODE_F7 = 64
SDL_SCANCODE_F8 = 65
SDL_SCANCODE_F9 = 66
SDL_SCANCODE_F10 = 67
SDL_SCANCODE_F11 = 68
SDL_SCANCODE_F12 = 69

SDL_SCANCODE_PRINTSCREEN = 70
SDL_SCANCODE_SCROLLLOCK = 71
SDL_SCANCODE_PAUSE = 72
SDL_SCANCODE_INSERT = 73
SDL_SCANCODE_HOME = 74
SDL_SCANCODE_PAGEUP = 75
SDL_SCANCODE_DELETE = 76
SDL_SCANCODE_END = 77
SDL_SCANCODE_PAGEDOWN = 78
SDL_SCANCODE_RIGHT = 79
SDL_SCANCODE_LEFT = 80
SDL_SCANCODE_DOWN = 81
SDL_SCANCODE_UP = 82

SDL_SCANCODE_NUMLOCKCLEAR = 83
SDL_SCANCODE_KP_DIVIDE = 84
SDL_SCANCODE_KP_MULTIPLY = 85
SDL_SCANCODE_KP_MINUS = 86
SDL_SCANCODE_KP_PLUS = 87
SDL_SCANCODE_KP_ENTER = 88
SDL_SCANCODE_KP_1 = 89
SDL_SCANCODE_KP_2 = 90
SDL_SCANCODE_KP_3 = 91
SDL_SCANCODE_KP_4 = 92
SDL_SCANCODE_KP_5 = 93
SDL_SCANCODE_KP_6 = 94
SDL_SCANCODE_KP_7 = 95
SDL_SCANCODE_KP_8 = 96
SDL_SCANCODE_KP_9 = 97
SDL_SCANCODE_KP_0 = 98
SDL_SCANCODE_KP_PERIOD = 99

SDL_SCANCODE_NONUSBACKSLASH = 100
SDL_SCANCODE_APPLICATION = 101
SDL_SCANCODE_POWER = 102
SDL_SCANCODE_KP_EQUALS = 103
SDL_SCANCODE_F13 = 104
SDL_SCANCODE_F14 = 105
SDL_SCANCODE_F15 = 106
SDL_SCANCODE_F16 = 107
SDL_SCANCODE_F17 = 108
SDL_SCANCODE_F18 = 109
SDL_SCANCODE_F19 = 110
SDL_SCANCODE_F20 = 111
SDL_SCANCODE_F21 = 112
SDL_SCANCODE_F22 = 113
SDL_SCANCODE_F23 = 114
SDL_SCANCODE_F24 = 115
SDL_SCANCODE_EXECUTE = 116
SDL_SCANCODE_HELP = 117
SDL_SCANCODE_MENU = 118
SDL_SCANCODE_SELECT = 119
SDL_SCANCODE_STOP = 120
SDL_SCANCODE_AGAIN = 121
SDL_SCANCODE_UNDO = 122
SDL_SCANCODE_CUT = 123
SDL_SCANCODE_COPY = 124
SDL_SCANCODE_PASTE = 125
SDL_SCANCODE_FIND = 126
SDL_SCANCODE_MUTE = 127
SDL_SCANCODE_VOLUMEUP = 128
SDL_SCANCODE_VOLUMEDOWN = 129
SDL_SCANCODE_KP_COMMA = 133
SDL_SCANCODE_KP_EQUALSAS400 = 134

SDL_SCANCODE_INTERNATIONAL1 = 135
SDL_SCANCODE_INTERNATIONAL2 = 136
SDL_SCANCODE_INTERNATIONAL3 = 137
SDL_SCANCODE_INTERNATIONAL4 = 138
SDL_SCANCODE_INTERNATIONAL5 = 139
SDL_SCANCODE_INTERNATIONAL6 = 140
SDL_SCANCODE_INTERNATIONAL7 = 141
SDL_SCANCODE_INTERNATIONAL8 = 142
SDL_SCANCODE_INTERNATIONAL9 = 143
SDL_SCANCODE_LANG1 = 144
SDL_SCANCODE_LANG2 = 145
SDL_SCANCODE_LANG3 = 146
SDL_SCANCODE_LANG4 = 147
SDL_SCANCODE_LANG5 = 148
SDL_SCANCODE_LANG6 = 149
SDL_SCANCODE_LANG7 = 150
SDL_SCANCODE_LANG8 = 151
SDL_SCANCODE_LANG9 = 152

SDL_SCANCODE_ALTERASE = 153
SDL_SCANCODE_SYSREQ = 154
SDL_SCANCODE_CANCEL = 155
SDL_SCANCODE_CLEAR = 156
SDL_SCANCODE_PRIOR = 157
SDL_SCANCODE_RETURN2 = 158
SDL_SCANCODE_SEPARATOR = 159
SDL_SCANCODE_OUT = 160
SDL_SCANCODE_OPER = 161
SDL_SCANCODE_CLEARAGAIN = 162
SDL_SCANCODE_CRSEL = 163
SDL_SCANCODE_EXSEL = 164

SDL_SCANCODE_KP_00 = 176
SDL_SCANCODE_KP_000 = 177
SDL_SCANCODE_THOUSANDSSEPARATOR = 178
SDL_SCANCODE_DECIMALSEPARATOR = 179
SDL_SCANCODE_CURRENCYUNIT = 180
SDL_SCANCODE_CURRENCYSUBUNIT = 181
SDL_SCANCODE_KP_LEFTPAREN = 182
SDL_SCANCODE_KP_RIGHTPAREN = 183
SDL_SCANCODE_KP_LEFTBRACE = 184
SDL_SCANCODE_KP_RIGHTBRACE = 185
SDL_SCANCODE_KP_TAB = 186
SDL_SCANCODE_KP_BACKSPACE = 187
SDL_SCANCODE_KP_A = 188
SDL_SCANCODE_KP_B = 189
SDL_SCANCODE_KP_C = 190
SDL_SCANCODE_KP_D = 191
SDL_SCANCODE_KP_E = 192
SDL_SCANCODE_KP_F = 193
SDL_SCANCODE_KP_XOR = 194
SDL_SCANCODE_KP_POWER = 195
SDL_SCANCODE_KP_PERCENT = 196
SDL_SCANCODE_KP_LESS = 197
SDL_SCANCODE_KP_GREATER = 198
SDL_SCANCODE_KP_AMPERSAND = 199
SDL_SCANCODE_KP_DBLAMPERSAND = 200
SDL_SCANCODE_KP_VERTICALBAR = 201
SDL_SCANCODE_KP_DBLVERTICALBAR = 202
SDL_SCANCODE_KP_COLON = 203
SDL_SCANCODE_KP_HASH = 204
SDL_SCANCODE_KP_SPACE = 205
SDL_SCANCODE_KP_AT = 206
SDL_SCANCODE_KP_EXCLAM = 207
SDL_SCANCODE_KP_MEMSTORE = 208
SDL_SCANCODE_KP_MEMRECALL = 209
SDL_SCANCODE_KP_MEMCLEAR = 210
SDL_SCANCODE_KP_MEMADD = 211
SDL_SCANCODE_KP_MEMSUBTRACT = 212
SDL_SCANCODE_KP_MEMMULTIPLY = 213
SDL_SCANCODE_KP_MEMDIVIDE = 214
SDL_SCANCODE_KP_PLUSMINUS = 215
SDL_SCANCODE_KP_CLEAR = 216
SDL_SCANCODE_KP_CLEARENTRY = 217
SDL_SCANCODE_KP_BINARY = 218
SDL_SCANCODE_KP_OCTAL = 219
SDL_SCANCODE_KP_DECIMAL = 220
SDL_SCANCODE_KP_HEXADECIMAL = 221

SDL_SCANCODE_LCTRL = 224
SDL_SCANCODE_LSHIFT = 225
SDL_SCANCODE_LALT = 226
SDL_SCANCODE_LGUI = 227
SDL_SCANCODE_RCTRL = 228
SDL_SCANCODE_RSHIFT = 229
SDL_SCANCODE_RALT = 230
SDL_SCANCODE_RGUI = 231

SDL_SCANCODE_MODE = 257

SDL_SCANCODE_SLEEP = 258
SDL_SCANCODE_WAKE = 259

SDL_SCANCODE_CHANNEL_INCREMENT = 260
SDL_SCANCODE_CHANNEL_DECREMENT = 261

SDL_SCANCODE_MEDIA_PLAY = 262
SDL_SCANCODE_MEDIA_PAUSE = 263
SDL_SCANCODE_MEDIA_RECORD = 264
SDL_SCANCODE_MEDIA_FAST_FORWARD = 265
SDL_SCANCODE_MEDIA_REWIND = 266
SDL_SCANCODE_MEDIA_NEXT_TRACK = 267
SDL_SCANCODE_MEDIA_PREVIOUS_TRACK = 268
SDL_SCANCODE_MEDIA_STOP = 269
SDL_SCANCODE_MEDIA_EJECT = 270
SDL_SCANCODE_MEDIA_PLAY_PAUSE = 271
SDL_SCANCODE_MEDIA_SELECT = 272

SDL_SCANCODE_AC_NEW = 273
SDL_SCANCODE_AC_OPEN = 274
SDL_SCANCODE_AC_CLOSE = 275
SDL_SCANCODE_AC_EXIT = 276
SDL_SCANCODE_AC_SAVE = 277
SDL_SCANCODE_AC_PRINT = 278
SDL_SCANCODE_AC_PROPERTIES = 279

SDL_SCANCODE_AC_SEARCH = 280
SDL_SCANCODE_AC_HOME = 281
SDL_SCANCODE_AC_BACK = 282
SDL_SCANCODE_AC_FORWARD = 283
SDL_SCANCODE_AC_STOP = 284
SDL_SCANCODE_AC_REFRESH = 285
SDL_SCANCODE_AC_BOOKMARKS = 286

SDL_SCANCODE_SOFTLEFT = 287
SDL_SCANCODE_SOFTRIGHT = 288
SDL_SCANCODE_CALL = 289
SDL_SCANCODE_ENDCALL = 290

SDL_SCANCODE_RESERVED = 400
SDL_SCANCODE_COUNT = 512


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

from .SDL_properties import SDL_PropertiesID
from .SDL_surface import SDL_Surface
from .SDL_pixels import SDL_PixelFormat
from .SDL_rect import SDL_Point, SDL_Rect
from .SDL_stdinc import SDL_FunctionPointer

SDL_SET_CURRENT_BINARY(SDL_BINARY)

SDL_DisplayID = ctypes.c_uint32
SDL_WindowID = ctypes.c_uint32

SDL_PROP_GLOBAL_VIDEO_WAYLAND_WL_DISPLAY_POINTER = "SDL.video.wayland.wl_display".encode()

SDL_SystemTheme = ctypes.c_int

SDL_SYSTEM_THEME_UNKNOWN = 0
SDL_SYSTEM_THEME_LIGHT = 1
SDL_SYSTEM_THEME_DARK = 2

class SDL_DisplayModeData(ctypes.c_void_p):
    ...

class SDL_DisplayMode(ctypes.Structure):
    _fields_ = [
        ("displayID", SDL_DisplayID),
        ("format", SDL_PixelFormat),
        ("w", ctypes.c_int),
        ("h", ctypes.c_int),
        ("pixel_density", ctypes.c_float),
        ("refresh_rate", ctypes.c_float),
        ("refresh_rate_numerator", ctypes.c_int),
        ("refresh_rate_denominator", ctypes.c_int),
        ("internal", ctypes.POINTER(SDL_DisplayModeData))
    ]

SDL_DisplayOrientation = ctypes.c_int

SDL_ORIENTATION_UNKNOWN = 0
SDL_ORIENTATION_LANDSCAPE = 1
SDL_ORIENTATION_LANDSCAPE_FLIPPED = 2
SDL_ORIENTATION_PORTRAIT = 3
SDL_ORIENTATION_PORTRAIT_FLIPPED = 4

class SDL_Window(ctypes.c_void_p):
    ...

SDL_WindowFlags = ctypes.c_uint64

SDL_WINDOW_FULLSCREEN = 0x0000000000000001
SDL_WINDOW_OPENGL = 0x0000000000000002
SDL_WINDOW_OCCLUDED = 0x0000000000000004
SDL_WINDOW_HIDDEN = 0x0000000000000008
SDL_WINDOW_BORDERLESS = 0x0000000000000010
SDL_WINDOW_RESIZABLE = 0x0000000000000020
SDL_WINDOW_MINIMIZED = 0x0000000000000040
SDL_WINDOW_MAXIMIZED = 0x0000000000000080
SDL_WINDOW_MOUSE_GRABBED = 0x0000000000000100
SDL_WINDOW_INPUT_FOCUS = 0x0000000000000200
SDL_WINDOW_MOUSE_FOCUS = 0x0000000000000400
SDL_WINDOW_EXTERNAL = 0x0000000000000800
SDL_WINDOW_MODAL = 0x0000000000001000
SDL_WINDOW_HIGH_PIXEL_DENSITY = 0x0000000000002000
SDL_WINDOW_MOUSE_CAPTURE = 0x0000000000004000
SDL_WINDOW_MOUSE_RELATIVE_MODE = 0x0000000000008000
SDL_WINDOW_ALWAYS_ON_TOP = 0x0000000000010000
SDL_WINDOW_UTILITY = 0x0000000000020000
SDL_WINDOW_TOOLTIP = 0x0000000000040000
SDL_WINDOW_POPUP_MENU = 0x0000000000080000
SDL_WINDOW_KEYBOARD_GRABBED = 0x0000000000100000
SDL_WINDOW_VULKAN = 0x0000000010000000
SDL_WINDOW_METAL = 0x0000000020000000
SDL_WINDOW_TRANSPARENT = 0x0000000040000000
SDL_WINDOW_NOT_FOCUSABLE = 0x0000000080000000

SDL_WINDOWPOS_UNDEFINED_MASK = 0x1FFF0000
SDL_WINDOWPOS_UNDEFINED_DISPLAY = lambda x: SDL_WINDOWPOS_UNDEFINED_MASK | x
SDL_WINDOWPOS_UNDEFINED = SDL_WINDOWPOS_UNDEFINED_DISPLAY(0)
SDL_WINDOWPOS_ISUNDEFINED = lambda x: (x & 0xFFFF0000) == SDL_WINDOWPOS_UNDEFINED_MASK

SDL_WINDOWPOS_CENTERED_MASK = 0x2FFF0000
SDL_WINDOWPOS_CENTERED_DISPLAY = lambda x: SDL_WINDOWPOS_CENTERED_MASK | x
SDL_WINDOWPOS_CENTERED = SDL_WINDOWPOS_CENTERED_DISPLAY(0)
SDL_WINDOWPOS_ISCENTERED = lambda x: (x & 0xFFFF0000) == SDL_WINDOWPOS_CENTERED_MASK

SDL_FlashOperation = ctypes.c_int

SDL_FLASH_CANCEL = 0
SDL_FLASH_BRIEFLY = 1
SDL_FLASH_UNTIL_FOCUSED = 2

class SDL_GLContextState(ctypes.c_void_p):
    ...

SDL_GLContext = ctypes.POINTER(SDL_GLContextState)

SDL_EGLDisplay = ctypes.c_void_p
SDL_EGLConfig = ctypes.c_void_p
SDL_EGLSurface = ctypes.c_void_p
SDL_EGLAttrib = ctypes.c_longlong
SDL_EGLint = ctypes.c_int

SDL_EGLAttribArrayCallback = ctypes.CFUNCTYPE(ctypes.POINTER(SDL_EGLAttrib), ctypes.c_void_p)
SDL_EGLIntArrayCallback = ctypes.CFUNCTYPE(ctypes.POINTER(SDL_EGLint), ctypes.c_void_p, SDL_EGLDisplay, SDL_EGLConfig)

SDL_GLAttr = ctypes.c_int

SDL_GL_RED_SIZE = 0
SDL_GL_GREEN_SIZE = 1
SDL_GL_BLUE_SIZE = 2
SDL_GL_ALPHA_SIZE = 3
SDL_GL_BUFFER_SIZE = 4
SDL_GL_DOUBLEBUFFER = 5
SDL_GL_DEPTH_SIZE = 6
SDL_GL_STENCIL_SIZE = 7
SDL_GL_ACCUM_RED_SIZE = 8
SDL_GL_ACCUM_GREEN_SIZE = 9
SDL_GL_ACCUM_BLUE_SIZE = 10
SDL_GL_ACCUM_ALPHA_SIZE = 11
SDL_GL_STEREO = 12
SDL_GL_MULTISAMPLEBUFFERS = 13
SDL_GL_MULTISAMPLESAMPLES = 14
SDL_GL_ACCELERATED_VISUAL = 15
SDL_GL_RETAINED_BACKING = 16
SDL_GL_CONTEXT_MAJOR_VERSION = 17
SDL_GL_CONTEXT_MINOR_VERSION = 18
SDL_GL_CONTEXT_FLAGS = 19
SDL_GL_CONTEXT_PROFILE_MASK = 20
SDL_GL_SHARE_WITH_CURRENT_CONTEXT = 21
SDL_GL_FRAMEBUFFER_SRGB_CAPABLE = 22
SDL_GL_CONTEXT_RELEASE_BEHAVIOR = 23
SDL_GL_CONTEXT_RESET_NOTIFICATION = 24
SDL_GL_CONTEXT_NO_ERROR = 25
SDL_GL_FLOATBUFFERS = 26
SDL_GL_EGL_PLATFORM = 27

SDL_GLProfile = ctypes.c_uint32

SDL_GL_CONTEXT_PROFILE_CORE = 0x0001
SDL_GL_CONTEXT_PROFILE_COMPATIBILITY = 0x0002
SDL_GL_CONTEXT_PROFILE_ES = 0x0004

SDL_GLContextFlag = ctypes.c_uint32

SDL_GL_CONTEXT_DEBUG_FLAG = 0x0001
SDL_GL_CONTEXT_FORWARD_COMPATIBLE_FLAG = 0x0002
SDL_GL_CONTEXT_ROBUST_ACCESS_FLAG = 0x0004
SDL_GL_CONTEXT_RESET_ISOLATION_FLAG = 0x0008

SDL_GLContextReleaseFlag = ctypes.c_uint32

SDL_GL_CONTEXT_RELEASE_BEHAVIOR_NONE = 0x0000
SDL_GL_CONTEXT_RELEASE_BEHAVIOR_FLUSH = 0x0001

SDL_GLContextResetNotification = ctypes.c_uint32

SDL_GL_CONTEXT_RESET_NO_NOTIFICATION = 0x0000
SDL_GL_CONTEXT_RESET_LOSE_CONTEXT = 0x0001

SDL_FUNC("SDL_GetNumVideoDrivers", ctypes.c_int)
SDL_FUNC("SDL_GetVideoDriver", ctypes.c_char_p, ctypes.c_int)
SDL_FUNC("SDL_GetCurrentVideoDriver", ctypes.c_char_p)
SDL_FUNC("SDL_GetSystemTheme", SDL_SystemTheme)
SDL_FUNC("SDL_GetDisplays", ctypes.POINTER(SDL_DisplayID), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetPrimaryDisplay", SDL_DisplayID)
SDL_FUNC("SDL_GetDisplayProperties", SDL_PropertiesID, SDL_DisplayID)

SDL_PROP_DISPLAY_HDR_ENABLED_BOOLEAN = "SDL.display.HDR_enabled".encode()
SDL_PROP_DISPLAY_KMSDRM_PANEL_ORIENTATION_NUMBER = "SDL.display.KMSDRM.panel_orientation".encode()

SDL_FUNC("SDL_GetDisplayName", ctypes.c_char_p, SDL_DisplayID)
SDL_FUNC("SDL_GetDisplayBounds", ctypes.c_bool, SDL_DisplayID, ctypes.POINTER(SDL_Rect))
SDL_FUNC("SDL_GetDisplayUsableBounds", ctypes.c_bool, SDL_DisplayID, ctypes.POINTER(SDL_Rect))

SDL_FUNC("SDL_GetNaturalDisplayOrientation", SDL_DisplayOrientation, SDL_DisplayID)
SDL_FUNC("SDL_GetCurrentDisplayOrientation", SDL_DisplayOrientation, SDL_DisplayID)

SDL_FUNC("SDL_GetDisplayContentScale", ctypes.c_float, SDL_DisplayID)

SDL_FUNC("SDL_GetFullscreenDisplayModes", ctypes.POINTER(ctypes.POINTER(SDL_DisplayMode)), SDL_DisplayID, ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetClosestFullscreenDisplayMode", ctypes.c_bool, SDL_DisplayID, ctypes.c_int, ctypes.c_int, ctypes.c_float, ctypes.c_bool, ctypes.POINTER(SDL_DisplayMode))

SDL_FUNC("SDL_GetDesktopDisplayMode", ctypes.POINTER(SDL_DisplayMode), SDL_DisplayID)
SDL_FUNC("SDL_GetCurrentDisplayMode", ctypes.POINTER(SDL_DisplayMode), SDL_DisplayID)

SDL_FUNC("SDL_GetDisplayForPoint", SDL_DisplayID, ctypes.POINTER(SDL_Point))
SDL_FUNC("SDL_GetDisplayForRect", SDL_DisplayID, ctypes.POINTER(SDL_Rect))
SDL_FUNC("SDL_GetDisplayForWindow", SDL_DisplayID, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_GetWindowPixelDensity", ctypes.c_float, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_GetWindowDisplayScale", ctypes.c_float, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowFullscreenMode", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_DisplayMode))
SDL_FUNC("SDL_GetWindowFullscreenMode", ctypes.POINTER(SDL_DisplayMode), ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_GetWindowICCProfile", ctypes.c_void_p, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_size_t))
SDL_FUNC("SDL_GetWindowPixelFormat", SDL_PixelFormat, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_GetWindows", ctypes.POINTER(ctypes.POINTER(SDL_Window)), ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_CreateWindow", ctypes.POINTER(SDL_Window), ctypes.c_char_p, ctypes.c_int, ctypes.c_int, SDL_WindowFlags)
SDL_FUNC("SDL_CreatePopupWindow", ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Window), ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, SDL_WindowFlags)
SDL_FUNC("SDL_CreateWindowWithProperties", ctypes.POINTER(SDL_Window), SDL_PropertiesID)

SDL_PROP_WINDOW_CREATE_ALWAYS_ON_TOP_BOOLEAN = "SDL.window.create.always_on_top".encode()
SDL_PROP_WINDOW_CREATE_BORDERLESS_BOOLEAN = "SDL.window.create.borderless".encode()
SDL_PROP_WINDOW_CREATE_FOCUSABLE_BOOLEAN = "SDL.window.create.focusable".encode()
SDL_PROP_WINDOW_CREATE_EXTERNAL_GRAPHICS_CONTEXT_BOOLEAN = "SDL.window.create.external_graphics_context".encode()
SDL_PROP_WINDOW_CREATE_FLAGS_NUMBER = "SDL.window.create.flags".encode()
SDL_PROP_WINDOW_CREATE_FULLSCREEN_BOOLEAN = "SDL.window.create.fullscreen".encode()
SDL_PROP_WINDOW_CREATE_HEIGHT_NUMBER = "SDL.window.create.height".encode()
SDL_PROP_WINDOW_CREATE_HIDDEN_BOOLEAN = "SDL.window.create.hidden".encode()
SDL_PROP_WINDOW_CREATE_HIGH_PIXEL_DENSITY_BOOLEAN = "SDL.window.create.high_pixel_density".encode()
SDL_PROP_WINDOW_CREATE_MAXIMIZED_BOOLEAN = "SDL.window.create.maximized".encode()
SDL_PROP_WINDOW_CREATE_MENU_BOOLEAN = "SDL.window.create.menu".encode()
SDL_PROP_WINDOW_CREATE_METAL_BOOLEAN = "SDL.window.create.metal".encode()
SDL_PROP_WINDOW_CREATE_MINIMIZED_BOOLEAN = "SDL.window.create.minimized".encode()
SDL_PROP_WINDOW_CREATE_MODAL_BOOLEAN = "SDL.window.create.modal".encode()
SDL_PROP_WINDOW_CREATE_MOUSE_GRABBED_BOOLEAN = "SDL.window.create.mouse_grabbed".encode()
SDL_PROP_WINDOW_CREATE_OPENGL_BOOLEAN = "SDL.window.create.opengl".encode()
SDL_PROP_WINDOW_CREATE_PARENT_POINTER = "SDL.window.create.parent".encode()
SDL_PROP_WINDOW_CREATE_RESIZABLE_BOOLEAN = "SDL.window.create.resizable".encode()
SDL_PROP_WINDOW_CREATE_TITLE_STRING = "SDL.window.create.title".encode()
SDL_PROP_WINDOW_CREATE_TRANSPARENT_BOOLEAN = "SDL.window.create.transparent".encode()
SDL_PROP_WINDOW_CREATE_TOOLTIP_BOOLEAN = "SDL.window.create.tooltip".encode()
SDL_PROP_WINDOW_CREATE_UTILITY_BOOLEAN = "SDL.window.create.utility".encode()
SDL_PROP_WINDOW_CREATE_VULKAN_BOOLEAN = "SDL.window.create.vulkan".encode()
SDL_PROP_WINDOW_CREATE_WIDTH_NUMBER = "SDL.window.create.width".encode()
SDL_PROP_WINDOW_CREATE_X_NUMBER = "SDL.window.create.x".encode()
SDL_PROP_WINDOW_CREATE_Y_NUMBER = "SDL.window.create.y".encode()
SDL_PROP_WINDOW_CREATE_COCOA_WINDOW_POINTER = "SDL.window.create.cocoa.window".encode()
SDL_PROP_WINDOW_CREATE_COCOA_VIEW_POINTER = "SDL.window.create.cocoa.view".encode()
SDL_PROP_WINDOW_CREATE_WAYLAND_SURFACE_ROLE_CUSTOM_BOOLEAN = "SDL.window.create.wayland.surface_role_custom".encode()
SDL_PROP_WINDOW_CREATE_WAYLAND_CREATE_EGL_WINDOW_BOOLEAN = "SDL.window.create.wayland.create_egl_window".encode()
SDL_PROP_WINDOW_CREATE_WAYLAND_WL_SURFACE_POINTER = "SDL.window.create.wayland.wl_surface".encode()
SDL_PROP_WINDOW_CREATE_WIN32_HWND_POINTER = "SDL.window.create.win32.hwnd".encode()
SDL_PROP_WINDOW_CREATE_WIN32_PIXEL_FORMAT_HWND_POINTER = "SDL.window.create.win32.pixel_format_hwnd".encode()
SDL_PROP_WINDOW_CREATE_X11_WINDOW_NUMBER = "SDL.window.create.x11.window".encode()

SDL_FUNC("SDL_GetWindowID", SDL_WindowID, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_GetWindowFromID", ctypes.POINTER(SDL_Window), SDL_WindowID)
SDL_FUNC("SDL_GetWindowParent", ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_GetWindowProperties", SDL_PropertiesID, ctypes.POINTER(SDL_Window))

SDL_PROP_WINDOW_SHAPE_POINTER = "SDL.window.shape".encode()
SDL_PROP_WINDOW_HDR_ENABLED_BOOLEAN = "SDL.window.HDR_enabled".encode()
SDL_PROP_WINDOW_SDR_WHITE_LEVEL_FLOAT = "SDL.window.SDR_white_level".encode()
SDL_PROP_WINDOW_HDR_HEADROOM_FLOAT = "SDL.window.HDR_headroom".encode()
SDL_PROP_WINDOW_ANDROID_WINDOW_POINTER = "SDL.window.android.window".encode()
SDL_PROP_WINDOW_ANDROID_SURFACE_POINTER = "SDL.window.android.surface".encode()
SDL_PROP_WINDOW_UIKIT_WINDOW_POINTER = "SDL.window.uikit.window".encode()
SDL_PROP_WINDOW_UIKIT_METAL_VIEW_TAG_NUMBER = "SDL.window.uikit.metal_view_tag".encode()
SDL_PROP_WINDOW_UIKIT_OPENGL_FRAMEBUFFER_NUMBER = "SDL.window.uikit.opengl.framebuffer".encode()
SDL_PROP_WINDOW_UIKIT_OPENGL_RENDERBUFFER_NUMBER = "SDL.window.uikit.opengl.renderbuffer".encode()
SDL_PROP_WINDOW_UIKIT_OPENGL_RESOLVE_FRAMEBUFFER_NUMBER = "SDL.window.uikit.opengl.resolve_framebuffer".encode()
SDL_PROP_WINDOW_KMSDRM_DEVICE_INDEX_NUMBER = "SDL.window.kmsdrm.dev_index".encode()
SDL_PROP_WINDOW_KMSDRM_DRM_FD_NUMBER = "SDL.window.kmsdrm.drm_fd".encode()
SDL_PROP_WINDOW_KMSDRM_GBM_DEVICE_POINTER = "SDL.window.kmsdrm.gbm_dev".encode()
SDL_PROP_WINDOW_COCOA_WINDOW_POINTER = "SDL.window.cocoa.window".encode()
SDL_PROP_WINDOW_COCOA_METAL_VIEW_TAG_NUMBER = "SDL.window.cocoa.metal_view_tag".encode()
SDL_PROP_WINDOW_OPENVR_OVERLAY_ID = "SDL.window.openvr.overlay_id".encode()
SDL_PROP_WINDOW_VIVANTE_DISPLAY_POINTER = "SDL.window.vivante.display".encode()
SDL_PROP_WINDOW_VIVANTE_WINDOW_POINTER = "SDL.window.vivante.window".encode()
SDL_PROP_WINDOW_VIVANTE_SURFACE_POINTER = "SDL.window.vivante.surface".encode()
SDL_PROP_WINDOW_WIN32_HWND_POINTER = "SDL.window.win32.hwnd".encode()
SDL_PROP_WINDOW_WIN32_HDC_POINTER = "SDL.window.win32.hdc".encode()
SDL_PROP_WINDOW_WIN32_INSTANCE_POINTER = "SDL.window.win32.instance".encode()
SDL_PROP_WINDOW_WAYLAND_DISPLAY_POINTER = "SDL.window.wayland.display".encode()
SDL_PROP_WINDOW_WAYLAND_SURFACE_POINTER = "SDL.window.wayland.surface".encode()
SDL_PROP_WINDOW_WAYLAND_VIEWPORT_POINTER = "SDL.window.wayland.viewport".encode()
SDL_PROP_WINDOW_WAYLAND_EGL_WINDOW_POINTER = "SDL.window.wayland.egl_window".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_SURFACE_POINTER = "SDL.window.wayland.xdg_surface".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_TOPLEVEL_POINTER = "SDL.window.wayland.xdg_toplevel".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_TOPLEVEL_EXPORT_HANDLE_STRING = "SDL.window.wayland.xdg_toplevel_export_handle".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_POPUP_POINTER = "SDL.window.wayland.xdg_popup".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_POSITIONER_POINTER = "SDL.window.wayland.xdg_positioner".encode()
SDL_PROP_WINDOW_X11_DISPLAY_POINTER = "SDL.window.x11.display".encode()
SDL_PROP_WINDOW_X11_SCREEN_NUMBER = "SDL.window.x11.screen".encode()
SDL_PROP_WINDOW_X11_WINDOW_NUMBER = "SDL.window.x11.window".encode()

SDL_FUNC("SDL_GetWindowFlags", SDL_WindowFlags, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowTitle", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_char_p)
SDL_FUNC("SDL_GetWindowTitle", ctypes.c_char_p, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowIcon", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Surface))

SDL_FUNC("SDL_SetWindowPosition", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_int, ctypes.c_int)
SDL_FUNC("SDL_GetWindowPosition", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_SetWindowSize", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_int, ctypes.c_int)
SDL_FUNC("SDL_GetWindowSize", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_GetWindowSafeArea", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Rect))

SDL_FUNC("SDL_SetWindowAspectRatio", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_float, ctypes.c_float)
SDL_FUNC("SDL_GetWindowAspectRatio", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float))

SDL_FUNC("SDL_GetWindowBordersSize", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetWindowSizeInPixels", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_SetWindowMinimumSize", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_int, ctypes.c_int)
SDL_FUNC("SDL_GetWindowMinimumSize", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_SetWindowMaximumSize", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_int, ctypes.c_int)
SDL_FUNC("SDL_GetWindowMaximumSize", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_SetWindowBordered", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)
SDL_FUNC("SDL_SetWindowResizable", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)
SDL_FUNC("SDL_SetWindowAlwaysOnTop", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)

SDL_FUNC("SDL_ShowWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_HideWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_RaiseWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_MaximizeWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_MinimizeWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_RestoreWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowFullscreen", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)
SDL_FUNC("SDL_SyncWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_WindowHasSurface", ctypes.c_bool, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_GetWindowSurface", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_SetWindowSurfaceVSync", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_int)

SDL_WINDOW_SURFACE_VSYNC_DISABLED = 0
SDL_WINDOW_SURFACE_VSYNC_ADAPTIVE = -1

SDL_FUNC("SDL_GetWindowSurfaceVSync", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_UpdateWindowSurface", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_UpdateWindowSurfaceRects", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Rect), ctypes.c_int)
SDL_FUNC("SDL_DestroyWindowSurface", ctypes.c_bool, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowKeyboardGrab", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)
SDL_FUNC("SDL_SetWindowMouseGrab", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)

SDL_FUNC("SDL_GetWindowKeyboardGrab", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_GetWindowMouseGrab", ctypes.c_bool, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_GetGrabbedWindow", ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowMouseRect", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Rect))
SDL_FUNC("SDL_GetWindowMouseRect", ctypes.POINTER(SDL_Rect), ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowOpacity", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_float)
SDL_FUNC("SDL_GetWindowOpacity", ctypes.c_float, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_SetWindowParent", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_SetWindowModal", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)
SDL_FUNC("SDL_SetWindowFocusable", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_bool)

SDL_FUNC("SDL_ShowWindowSystemMenu", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.c_int, ctypes.c_int)

SDL_HitTestResult = ctypes.c_int

SDL_HITTEST_NORMAL = 0
SDL_HITTEST_DRAGGABLE = 1
SDL_HITTEST_RESIZE_TOPLEFT = 2
SDL_HITTEST_RESIZE_TOP = 3
SDL_HITTEST_RESIZE_TOPRIGHT = 4
SDL_HITTEST_RESIZE_RIGHT = 5
SDL_HITTEST_RESIZE_BOTTOMRIGHT = 6
SDL_HITTEST_RESIZE_BOTTOM = 7
SDL_HITTEST_RESIZE_BOTTOMLEFT = 8
SDL_HITTEST_RESIZE_LEFT = 9

SDL_HitTest = ctypes.CFUNCTYPE(SDL_HitTestResult, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Point), ctypes.c_void_p)

SDL_FUNC("SDL_SetWindowHitTest", ctypes.c_bool, ctypes.POINTER(SDL_Window), SDL_HitTest, ctypes.c_void_p)
SDL_FUNC("SDL_SetWindowShape", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Surface))

SDL_FUNC("SDL_FlashWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_FlashOperation))
SDL_FUNC("SDL_DestroyWindow", None, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_ScreenSaverEnabled", ctypes.c_bool)
SDL_FUNC("SDL_EnableScreenSaver", ctypes.c_bool)
SDL_FUNC("SDL_DisableScreenSaver", ctypes.c_bool)

SDL_FUNC("SDL_GL_LoadLibrary", ctypes.c_bool, ctypes.c_char_p)
SDL_FUNC("SDL_GL_GetProcAddress", SDL_FunctionPointer, ctypes.c_char_p)
SDL_FUNC("SDL_EGL_GetProcAddress", SDL_FunctionPointer, ctypes.c_char_p)
SDL_FUNC("SDL_GL_UnloadLibrary", None)

SDL_FUNC("SDL_GL_ExtensionSupported", ctypes.c_bool, ctypes.c_char_p)

SDL_FUNC("SDL_GL_ResetAttributes", None)
SDL_FUNC("SDL_GL_SetAttribute", ctypes.c_bool, SDL_GLAttr, ctypes.c_int)
SDL_FUNC("SDL_GL_GetAttribute", ctypes.c_bool, SDL_GLAttr, ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_GL_CreateContext", SDL_GLContext, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_GL_MakeCurrent", ctypes.c_bool, ctypes.POINTER(SDL_Window), SDL_GLContext)

SDL_FUNC("SDL_GL_GetCurrentWindow", ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_GL_GetCurrentContext", SDL_GLContext)

SDL_FUNC("SDL_EGL_GetCurrentDisplay", SDL_EGLDisplay)
SDL_FUNC("SDL_EGL_GetCurrentConfig", SDL_EGLConfig)
SDL_FUNC("SDL_EGL_GetWindowSurface", SDL_EGLSurface, ctypes.POINTER(SDL_Window))

SDL_FUNC("SDL_EGL_SetAttributeCallbacks", None, SDL_EGLAttribArrayCallback, SDL_EGLIntArrayCallback, SDL_EGLIntArrayCallback, ctypes.c_void_p)

SDL_FUNC("SDL_GL_SetSwapInterval", ctypes.c_bool, ctypes.c_int)
SDL_FUNC("SDL_GL_GetSwapInterval", ctypes.c_bool, ctypes.POINTER(ctypes.c_int))

SDL_FUNC("SDL_GL_SwapWindow", ctypes.c_bool, ctypes.POINTER(SDL_Window))
SDL_FUNC("SDL_GL_DestroyContext", ctypes.c_bool, SDL_GLContext)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_IMAGE_BINARY

from .SDL_surface import SDL_Surface
from .SDL_render import SDL_Texture, SDL_Renderer
from .SDL_iostream import SDL_IOStream
from .SDL_version import SDL_VERSIONNUM

SDL_SET_CURRENT_BINARY(SDL_IMAGE_BINARY)

SDL_IMAGE_MAJOR_VERSION = 3
SDL_IMAGE_MINOR_VERSION = 2
SDL_IMAGE_MICRO_VERSION = 1

SDL_IMAGE_VERSION = \
    SDL_VERSIONNUM(SDL_IMAGE_MAJOR_VERSION, SDL_IMAGE_MINOR_VERSION, SDL_IMAGE_MICRO_VERSION)

SDL_IMAGE_VERSION_ATLEAST = lambda x, y, z: \
    (SDL_IMAGE_MAJOR_VERSION >= x) and \
    (SDL_IMAGE_MAJOR_VERSION > x or SDL_IMAGE_MINOR_VERSION >= y) and \
    (SDL_IMAGE_MAJOR_VERSION > x or SDL_IMAGE_MINOR_VERSION > y or SDL_IMAGE_MICRO_VERSION >= z)

SDL_FUNC("IMG_Version", ctypes.c_int)

SDL_FUNC("IMG_LoadTyped_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_char_p)
SDL_FUNC("IMG_Load", ctypes.POINTER(SDL_Surface), ctypes.c_char_p)
SDL_FUNC("IMG_Load_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream), ctypes.c_bool)

SDL_FUNC("IMG_LoadTexture", ctypes.POINTER(SDL_Texture), ctypes.POINTER(SDL_Renderer), ctypes.c_char_p)
SDL_FUNC("IMG_LoadTexture_IO", ctypes.POINTER(SDL_Texture), ctypes.POINTER(SDL_Renderer), ctypes.POINTER(SDL_IOStream), ctypes.c_bool)
SDL_FUNC("IMG_LoadTextureTyped_IO", ctypes.POINTER(SDL_Texture), ctypes.POINTER(SDL_Renderer), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_char_p)

SDL_FUNC("IMG_isAVIF", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isICO", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isCUR", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isBMP", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isGIF", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isJPG", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isJXL", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isLBM", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isPCX", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isPNG", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isPNM", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isSVG", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isQOI", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isTIF", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isXCF", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isXPM", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isXV", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_isWEBP", ctypes.c_bool, ctypes.POINTER(SDL_IOStream))

SDL_FUNC("IMG_LoadAVIF_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadICO_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadCUR_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadBMP_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadGIF_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadJPG_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadJXL_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadLBM_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadPCX_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadPNG_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadPNM_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadSVG_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadQOI_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadTGA_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadTIF_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadXCF_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadXPM_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadXV_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadWEBP_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadSizedSVG_IO", ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream), ctypes.c_int, ctypes.c_int)

SDL_FUNC("IMG_ReadXPMFromArray", ctypes.POINTER(SDL_Surface), ctypes.POINTER(ctypes.c_char_p))
SDL_FUNC("IMG_ReadXPMFromArrayToRGB888", ctypes.POINTER(SDL_Surface), ctypes.POINTER(ctypes.c_char_p))

SDL_FUNC("IMG_SaveAVIF", ctypes.c_bool, ctypes.POINTER(SDL_Surface), ctypes.c_char_p, ctypes.c_int)
SDL_FUNC("IMG_SaveAVIF_IO", ctypes.c_bool, ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_int)
SDL_FUNC("IMG_SavePNG", ctypes.c_bool, ctypes.POINTER(SDL_Surface), ctypes.c_char_p)
SDL_FUNC("IMG_SavePNG_IO", ctypes.c_bool, ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream), ctypes.c_bool)
SDL_FUNC("IMG_SaveJPG", ctypes.c_bool, ctypes.POINTER(SDL_Surface), ctypes.c_char_p, ctypes.c_int)
SDL_FUNC("IMG_SaveJPG_IO", ctypes.c_bool, ctypes.POINTER(SDL_Surface), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_int)

class IMG_Animation(ctypes.Structure):
    _fields_ = [
        ("w", ctypes.c_int),
        ("h", ctypes.c_int),
        ("count", ctypes.c_int),
        ("frames", ctypes.POINTER(ctypes.POINTER(SDL_Surface))),
        ("delays", ctypes.POINTER(ctypes.c_int))
    ]

SDL_FUNC("IMG_LoadAnimation", ctypes.POINTER(IMG_Animation), ctypes.c_char_p)
SDL_FUNC("IMG_LoadAnimation_IO", ctypes.POINTER(IMG_Animation), ctypes.POINTER(SDL_IOStream), ctypes.c_bool)
SDL_FUNC("IMG_LoadAnimationTyped_IO", ctypes.POINTER(IMG_Animation), ctypes.POINTER(SDL_IOStream), ctypes.c_bool, ctypes.c_char_p)
SDL_FUNC("IMG_FreeAnimation", None, ctypes.POINTER(IMG_Animation))
SDL_FUNC("IMG_LoadGIFAnimation_IO", ctypes.POINTER(IMG_Animation), ctypes.POINTER(SDL_IOStream))
SDL_FUNC("IMG_LoadWEBPAnimation_IO", ctypes.POINTER(IMG_Animation), ctypes.POINTER(SDL_IOStream))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_GET_BINARY, SDL_BINARY

from .SDL_atomic import SDL_AtomicInt
from .SDL_stdinc import SDL_FunctionPointer
from .SDL_properties import SDL_PropertiesID

SDL_SET_CURRENT_BINARY(SDL_BINARY)

class SDL_Thread(ctypes.c_void_p):
    ...

SDL_ThreadID = ctypes.c_uint64
SDL_TLSID = SDL_AtomicInt

SDL_ThreadPriority = ctypes.c_int

SDL_THREAD_PRIORITY_LOW = 0
SDL_THREAD_PRIORITY_NORMAL = 1
SDL_THREAD_PRIORITY_HIGH = 2
SDL_THREAD_PRIORITY_TIME_CRITICAL = 3

SDL_ThreadState = ctypes.c_int

SDL_THREAD_UNKNOWN = 0
SDL_THREAD_ALIVE = 1
SDL_THREAD_DETACHED = 2
SDL_THREAD_COMPLETE = 3

SDL_ThreadFunction = ctypes.CFUNCTYPE(ctypes.c_int, ctypes.c_void_p)

SDL_BeginThreadFunction = SDL_FunctionPointer(0)
SDL_EndThreadFunction = SDL_FunctionPointer(0)

SDL_FUNC("SDL_CreateThreadRuntime", ctypes.POINTER(SDL_Thread), SDL_ThreadFunction, ctypes.c_char_p, ctypes.c_void_p, SDL_FunctionPointer, SDL_FunctionPointer)
SDL_FUNC("SDL_CreateThreadWithPropertiesRuntime", ctypes.POINTER(SDL_Thread), SDL_PropertiesID, SDL_FunctionPointer, SDL_FunctionPointer)

SDL_CreateThread = lambda fn, name, data: \
    SDL_GET_BINARY(SDL_BINARY).SDL_CreateThreadRuntime(fn, name, data, SDL_BeginThreadFunction, SDL_EndThreadFunction)

SDL_CreateThreadWithProperties = lambda props: \
    SDL_GET_BINARY(SDL_BINARY).SDL_CreateThreadWithPropertiesRuntime(props, SDL_BeginThreadFunction, SDL_EndThreadFunction)

SDL_PROP_THREAD_CREATE_ENTRY_FUNCTION_POINTER = "SDL.thread.create.entry_function".encode()
SDL_PROP_THREAD_CREATE_NAME_STRING = "SDL.thread.create.name".encode()
SDL_PROP_THREAD_CREATE_USERDATA_POINTER = "SDL.thread.create.userdata".encode()
SDL_PROP_THREAD_CREATE_STACKSIZE_NUMBER = "SDL.thread.create.stacksize".encode()

SDL_FUNC("SDL_GetThreadName", ctypes.c_char_p, ctypes.POINTER(SDL_Thread))
SDL_FUNC("SDL_GetCurrentThreadID", SDL_ThreadID)
SDL_FUNC("SDL_GetThreadID", SDL_ThreadID, ctypes.POINTER(SDL_Thread))
SDL_FUNC("SDL_SetCurrentThreadPriority", ctypes.c_bool, SDL_ThreadPriority)
SDL_FUNC("SDL_WaitThread", None, ctypes.POINTER(SDL_Thread), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_GetThreadState", SDL_ThreadState, ctypes.POINTER(SDL_Thread))
SDL_FUNC("SDL_DetachThread", None, ctypes.POINTER(SDL_Thread))
SDL_FUNC("SDL_GetTLS", ctypes.c_void_p, ctypes.POINTER(SDL_TLSID))

SDL_TLSDestructorCallback = ctypes.CFUNCTYPE(None, ctypes.c_void_p)

SDL_FUNC("SDL_SetTLS", ctypes.c_bool, ctypes.POINTER(SDL_TLSID), ctypes.c_void_p, SDL_TLSDestructorCallback)
SDL_FUNC("SDL_CleanupTLS", None)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

SDL_SET_CURRENT_BINARY(SDL_BINARY)

SDL_FUNC("SDL_SetClipboardText", ctypes.c_bool, ctypes.c_char_p)
SDL_FUNC("SDL_GetClipboardText", ctypes.c_char_p)
SDL_FUNC("SDL_HasClipboardText", ctypes.c_bool)

SDL_FUNC("SDL_SetPrimarySelectionText", ctypes.c_bool, ctypes.c_char_p)
SDL_FUNC("SDL_GetPrimarySelectionText", ctypes.c_char_p)
SDL_FUNC("SDL_HasPrimarySelectionText", ctypes.c_bool)

SDL_ClipboardDataCallback = ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.c_char_p, ctypes.POINTER(ctypes.c_size_t))
SDL_ClipboardCleanupCallback = ctypes.CFUNCTYPE(None, ctypes.c_void_p)

SDL_FUNC("SDL_SetClipboardData", ctypes.c_bool, SDL_ClipboardDataCallback, SDL_ClipboardCleanupCallback, ctypes.c_void_p, ctypes.POINTER(ctypes.c_char_p), ctypes.c_size_t)
SDL_FUNC("SDL_ClearClipboardData", ctypes.c_bool)
SDL_FUNC("SDL_GetClipboardData", ctypes.c_void_p, ctypes.c_char_p, ctypes.POINTER(ctypes.c_size_t))
SDL_FUNC("SDL_HasClipboardData", ctypes.c_bool, ctypes.c_char_p)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

from .SDL_surface import SDL_Surface

SDL_SET_CURRENT_BINARY(SDL_BINARY)

class SDL_Tray(ctypes.c_void_p):
    ...

class SDL_TrayMenu(ctypes.c_void_p):
    ...

class SDL_TrayEntry(ctypes.c_void_p):
    ...

SDL_TrayEntryFlags = ctypes.c_uint32

SDL_TRAYENTRY_BUTTON = 0x00000001
SDL_TRAYENTRY_CHECKBOX = 0x00000002
SDL_TRAYENTRY_SUBMENU = 0x00000004
SDL_TRAYENTRY_DISABLED = 0x80000000
SDL_TRAYENTRY_CHECKED = 0x40000000

SDL_TrayCallback = ctypes.CFUNCTYPE(None, ctypes.c_void_p, ctypes.POINTER(SDL_TrayEntry))

SDL_FUNC("SDL_CreateTray", ctypes.POINTER(SDL_Tray), ctypes.POINTER(SDL_Surface), ctypes.c_char_p)
SDL_FUNC("SDL_SetTrayIcon", None, ctypes.POINTER(SDL_Tray), ctypes.POINTER(SDL_Surface))
SDL_FUNC("SDL_SetTrayTooltip", None, ctypes.POINTER(SDL_Tray), ctypes.c_char_p)
SDL_FUNC("SDL_CreateTrayMenu", ctypes.POINTER(SDL_TrayMenu), ctypes.POINTER(SDL_Tray))
SDL_FUNC("SDL_CreateTraySubmenu", ctypes.POINTER(SDL_TrayMenu), ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_GetTrayMenu", ctypes.POINTER(SDL_TrayMenu), ctypes.POINTER(SDL_Tray))
SDL_FUNC("SDL_GetTraySubmenu", ctypes.POINTER(SDL_TrayMenu), ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_GetTrayEntries", ctypes.POINTER(ctypes.POINTER(SDL_TrayEntry)), ctypes.POINTER(SDL_TrayMenu), ctypes.POINTER(ctypes.c_int))
SDL_FUNC("SDL_RemoveTrayEntry", None, ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_InsertTrayEntryAt", ctypes.POINTER(SDL_TrayEntry), ctypes.POINTER(SDL_TrayMenu), ctypes.c_int, ctypes.c_char_p, SDL_TrayEntryFlags)
SDL_FUNC("SDL_SetTrayEntryLabel", None, ctypes.POINTER(SDL_TrayEntry), ctypes.c_char_p)
SDL_FUNC("SDL_GetTrayEntryLabel", ctypes.c_char_p, ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_SetTrayEntryChecked", None, ctypes.POINTER(SDL_TrayEntry), ctypes.c_bool)
SDL_FUNC("SDL_GetTrayEntryChecked", ctypes.c_bool, ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_SetTrayEntryEnabled", None, ctypes.POINTER(SDL_TrayEntry), ctypes.c_bool)
SDL_FUNC("SDL_GetTrayEntryEnabled", ctypes.c_bool, ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_SetTrayEntryCallback", None, ctypes.POINTER(SDL_TrayEntry), SDL_TrayCallback, ctypes.c_void_p)
SDL_FUNC("SDL_ClickTrayEntry", None, ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_DestroyTray", None, ctypes.POINTER(SDL_Tray))
SDL_FUNC("SDL_GetTrayEntryParent", ctypes.POINTER(SDL_TrayMenu), ctypes.POINTER(SDL_TrayEntry))
SDL_FUNC("SDL_GetTrayMenuParentEntry", ctypes.POINTER(SDL_TrayEntry), ctypes.POINTER(SDL_TrayMenu))
SDL_FUNC("SDL_GetTrayMenuParentTray", ctypes.POINTER(SDL_Tray), ctypes.POINTER(SDL_TrayMenu))
SDL_FUNC("SDL_UpdateTrays", None)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

SDL_SET_CURRENT_BINARY(SDL_BINARY)

SDL_PowerState = ctypes.c_int

SDL_POWERSTATE_ERROR = -1
SDL_POWERSTATE_UNKNOWN = 0
SDL_POWERSTATE_ON_BATTERY = 1
SDL_POWERSTATE_NO_BATTERY = 2
SDL_POWERSTATE_CHARGING = 3
SDL_POWERSTATE_CHARGED = 4

SDL_FUNC("SDL_GetPowerInfo", SDL_PowerState, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

SDL_SET_CURRENT_BINARY(SDL_BINARY)

SDL_MAJOR_VERSION = 3
SDL_MINOR_VERSION = 2
SDL_MICRO_VERSION = 5

SDL_VERSIONNUM = lambda major, minor, patch: \
    major * 1000000 + minor * 1000 + patch

SDL_VERSIONNUM_MAJOR = lambda version: int(version / 1000000)
SDL_VERSIONNUM_MINOR = lambda version: int(version / 1000) % 1000
SDL_VERSIONNUM_MICRO = lambda version: version % 1000

SDL_VERSION = SDL_VERSIONNUM(SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_MICRO_VERSION)
SDL_VERSION_ATLEAST = lambda x, y, z: SDL_VERSION >= SDL_VERSIONNUM(x, y, z)

SDL_FUNC("SDL_GetVersion", ctypes.c_int)
SDL_FUNC("SDL_GetRevision", ctypes.c_char_p)


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

SDL_SET_CURRENT_BINARY(SDL_BINARY)

class SDL_Locale(ctypes.Structure):
    _fields_ = [
        ("language", ctypes.c_char_p),
        ("country", ctypes.c_char_p)
    ]

SDL_FUNC("SDL_GetPreferredLocales", ctypes.POINTER(ctypes.POINTER(SDL_Locale)), ctypes.POINTER(ctypes.c_int))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_GET_BINARY, SDL_RTF_BINARY, SDL_BINARY

from .SDL_pixels import SDL_Color
from .SDL_render import SDL_Renderer, SDL_Texture
from .SDL_version import SDL_VERSIONNUM
from .SDL_iostream import SDL_IOStream
from .SDL_rect import SDL_Rect

SDL_SET_CURRENT_BINARY(SDL_RTF_BINARY)

SDL_RTF_MAJOR_VERSION = 3
SDL_RTF_MINOR_VERSION = 0
SDL_RTF_MICRO_VERSION = 0

SDL_RTF_VERSION = \
    SDL_VERSIONNUM(SDL_RTF_MAJOR_VERSION, SDL_RTF_MINOR_VERSION, SDL_RTF_MICRO_VERSION)

SDL_RTF_VERSION_ATLEAST = lambda x, y, z: \
    (SDL_RTF_MAJOR_VERSION >= x) and \
    (SDL_RTF_MAJOR_VERSION > x or SDL_RTF_MINOR_VERSION >= y) and \
    (SDL_RTF_MAJOR_VERSION > x or SDL_RTF_MINOR_VERSION > y or SDL_RTF_MICRO_VERSION >= z)

SDL_FUNC("RTF_Version", ctypes.c_int)

class RTF_Context(ctypes.c_void_p):
    ...

RTF_FontFamily = ctypes.c_int

RTF_FontDefault = 0
RTF_FontRoman = 1
RTF_FontSwiss = 2
RTF_FontModern = 3
RTF_FontScript = 4
RTF_FontDecor = 5
RTF_FontTech = 6
RTF_FontBidi = 7

RTF_FontStyle = ctypes.c_int

RTF_FontNormal = 0x00
RTF_FontBold = 0x01
RTF_FontItalic = 0x02
RTF_FontUnderline = 0x04

RTF_FONT_ENGINE_VERSION = 1

class RTF_FontEngine(ctypes.Structure):
    _fields_ = [
        ("version", ctypes.c_int),
        ("CreateFont", ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.c_char_p, RTF_FontFamily, ctypes.c_int, ctypes.c_int, ctypes.c_int)),
        ("GetLineSpacing", ctypes.CFUNCTYPE(ctypes.c_int, ctypes.c_void_p)),
        ("GetCharacterOffsets", ctypes.CFUNCTYPE(ctypes.c_int, ctypes.c_void_p, ctypes.c_char_p, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.c_int)),
        ("RenderText", ctypes.CFUNCTYPE(ctypes.POINTER(SDL_Texture), ctypes.c_void_p, ctypes.POINTER(SDL_Renderer), ctypes.c_char_p, SDL_Color)),
        ("FreeFont", ctypes.CFUNCTYPE(None, ctypes.c_void_p))
    ]

SDL_FUNC("RTF_CreateContext", ctypes.POINTER(RTF_Context), ctypes.POINTER(SDL_Renderer), ctypes.POINTER(RTF_FontEngine))
SDL_FUNC("RTF_Load", ctypes.c_int, ctypes.POINTER(RTF_Context), ctypes.c_char_p)
SDL_FUNC("RTF_Load_IO", ctypes.c_int, ctypes.POINTER(RTF_Context), ctypes.POINTER(SDL_IOStream), ctypes.c_int)
SDL_FUNC("RTF_GetTitle", ctypes.c_char_p, ctypes.POINTER(RTF_Context))
SDL_FUNC("RTF_GetSubject", ctypes.c_char_p, ctypes.POINTER(RTF_Context))
SDL_FUNC("RTF_GetAuthor", ctypes.c_char_p, ctypes.POINTER(RTF_Context))
SDL_FUNC("RTF_GetHeight", ctypes.c_int, ctypes.POINTER(RTF_Context), ctypes.c_int)
SDL_FUNC("RTF_Render", None, ctypes.POINTER(RTF_Context), ctypes.POINTER(SDL_Rect), ctypes.c_int)
SDL_FUNC("RTF_FreeContext", None, ctypes.POINTER(RTF_Context))

RTF_SetError = SDL_GET_BINARY(SDL_BINARY).SDL_SetError
RTF_GetError = SDL_GET_BINARY(SDL_BINARY).SDL_GetError


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

from .SDL_video import SDL_Window, SDL_WindowID, SDL_DisplayID
from .SDL_sensor import SDL_SensorID
from .SDL_joystick import SDL_JoystickID
from .SDL_keyboard import SDL_KeyboardID
from .SDL_keycode import SDL_Keycode, SDL_Keymod
from .SDL_scancode import SDL_Scancode
from .SDL_mouse import SDL_MouseID, SDL_MouseWheelDirection, SDL_MouseButtonFlags
from .SDL_power import SDL_PowerState
from .SDL_audio import SDL_AudioDeviceID
from .SDL_camera import SDL_CameraID
from .SDL_pen import SDL_PenID, SDL_PenInputFlags, SDL_PenAxis
from .SDL_touch import SDL_TouchID, SDL_FingerID

SDL_SET_CURRENT_BINARY(SDL_BINARY)

SDL_EventType = ctypes.c_int

SDL_EVENT_FIRST = 0x0
SDL_EVENT_QUIT = 0x100
SDL_EVENT_TERMINATING = 0x101
SDL_EVENT_LOW_MEMORY = 0x102
SDL_EVENT_WILL_ENTER_BACKGROUND = 0x103
SDL_EVENT_DID_ENTER_BACKGROUND = 0x104
SDL_EVENT_WILL_ENTER_FOREGROUND = 0x105
SDL_EVENT_DID_ENTER_FOREGROUND = 0x106
SDL_EVENT_LOCALE_CHANGED = 0x107
SDL_EVENT_SYSTEM_THEME_CHANGED = 0x108

SDL_EVENT_DISPLAY_ORIENTATION = 0x151
SDL_EVENT_DISPLAY_ADDED = 0x152
SDL_EVENT_DISPLAY_REMOVED = 0x153
SDL_EVENT_DISPLAY_MOVED = 0x154
SDL_EVENT_DISPLAY_DESKTOP_MODE_CHANGED = 0x155
SDL_EVENT_DISPLAY_CURRENT_MODE_CHANGED = 0x156
SDL_EVENT_DISPLAY_CONTENT_SCALE_CHANGED = 0x157
SDL_EVENT_DISPLAY_FIRST = SDL_EVENT_DISPLAY_ORIENTATION
SDL_EVENT_DISPLAY_LAST = SDL_EVENT_DISPLAY_CONTENT_SCALE_CHANGED

SDL_EVENT_WINDOW_SHOWN = 0x202
SDL_EVENT_WINDOW_HIDDEN = 0x203
SDL_EVENT_WINDOW_EXPOSED = 0x204
SDL_EVENT_WINDOW_MOVED = 0x205
SDL_EVENT_WINDOW_RESIZED = 0x206
SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED = 0x207
SDL_EVENT_WINDOW_METAL_VIEW_RESIZED = 0x208
SDL_EVENT_WINDOW_MINIMIZED = 0x209
SDL_EVENT_WINDOW_MAXIMIZED = 0x20A
SDL_EVENT_WINDOW_RESTORED = 0x20B
SDL_EVENT_WINDOW_MOUSE_ENTER = 0x20C
SDL_EVENT_WINDOW_MOUSE_LEAVE = 0x20D
SDL_EVENT_WINDOW_FOCUS_GAINED = 0x20E
SDL_EVENT_WINDOW_FOCUS_LOST = 0x20F
SDL_EVENT_WINDOW_CLOSE_REQUESTED = 0x210
SDL_EVENT_WINDOW_HIT_TEST = 0x211
SDL_EVENT_WINDOW_ICCPROF_CHANGED = 0x212
SDL_EVENT_WINDOW_DISPLAY_CHANGED = 0x213
SDL_EVENT_WINDOW_DISPLAY_SCALE_CHANGED = 0x214
SDL_EVENT_WINDOW_SAFE_AREA_CHANGED = 0x215
SDL_EVENT_WINDOW_OCCLUDED = 0x216
SDL_EVENT_WINDOW_ENTER_FULLSCREEN = 0x217
SDL_EVENT_WINDOW_LEAVE_FULLSCREEN = 0x218
SDL_EVENT_WINDOW_DESTROYED = 0x219

SDL_EVENT_WINDOW_HDR_STATE_CHANGED = 0x21A
SDL_EVENT_WINDOW_FIRST = SDL_EVENT_WINDOW_SHOWN
SDL_EVENT_WINDOW_LAST = SDL_EVENT_WINDOW_HDR_STATE_CHANGED

SDL_EVENT_KEY_DOWN = 0x300
SDL_EVENT_KEY_UP = 0x301
SDL_EVENT_TEXT_EDITING = 0x302
SDL_EVENT_TEXT_INPUT = 0x303
SDL_EVENT_KEYMAP_CHANGED = 0x304

SDL_EVENT_KEYBOARD_ADDED = 0x305
SDL_EVENT_KEYBOARD_REMOVED = 0x306
SDL_EVENT_TEXT_EDITING_CANDIDATES = 0x307

SDL_EVENT_MOUSE_MOTION = 0x400
SDL_EVENT_MOUSE_BUTTON_DOWN = 0x401
SDL_EVENT_MOUSE_BUTTON_UP = 0x402
SDL_EVENT_MOUSE_WHEEL = 0x403
SDL_EVENT_MOUSE_ADDED = 0x404
SDL_EVENT_MOUSE_REMOVED = 0x405

SDL_EVENT_JOYSTICK_AXIS_MOTION = 0x600
SDL_EVENT_JOYSTICK_BALL_MOTION = 0x601
SDL_EVENT_JOYSTICK_HAT_MOTION = 0x602
SDL_EVENT_JOYSTICK_BUTTON_DOWN = 0x603
SDL_EVENT_JOYSTICK_BUTTON_UP = 0x604
SDL_EVENT_JOYSTICK_ADDED = 0x605
SDL_EVENT_JOYSTICK_REMOVED = 0x606
SDL_EVENT_JOYSTICK_BATTERY_UPDATED = 0x607
SDL_EVENT_JOYSTICK_UPDATE_COMPLETE = 0x608

SDL_EVENT_GAMEPAD_AXIS_MOTION = 0x650
SDL_EVENT_GAMEPAD_BUTTON_DOWN = 0x651
SDL_EVENT_GAMEPAD_BUTTON_UP = 0x652
SDL_EVENT_GAMEPAD_ADDED = 0x653
SDL_EVENT_GAMEPAD_REMOVED = 0x654
SDL_EVENT_GAMEPAD_REMAPPED = 0x655
SDL_EVENT_GAMEPAD_TOUCHPAD_DOWN = 0x656
SDL_EVENT_GAMEPAD_TOUCHPAD_MOTION = 0x657
SDL_EVENT_GAMEPAD_TOUCHPAD_UP = 0x658
SDL_EVENT_GAMEPAD_SENSOR_UPDATE = 0x659
SDL_EVENT_GAMEPAD_UPDATE_COMPLETE = 0x65A
SDL_EVENT_GAMEPAD_STEAM_HANDLE_UPDATED = 0x65B

SDL_EVENT_FINGER_DOWN = 0x700
SDL_EVENT_FINGER_UP = 0x701
SDL_EVENT_FINGER_MOTION = 0x702
SDL_EVENT_FINGER_CANCELED = 0x703

SDL_EVENT_CLIPBOARD_UPDATE = 0x900

SDL_EVENT_DROP_FILE = 0x1000
SDL_EVENT_DROP_TEXT = 0x1001
SDL_EVENT_DROP_BEGIN = 0x1002
SDL_EVENT_DROP_COMPLETE = 0x1003
SDL_EVENT_DROP_POSITION = 0x1004

SDL_EVENT_AUDIO_DEVICE_ADDED = 0x1100
SDL_EVENT_AUDIO_DEVICE_REMOVED = 0x1101
SDL_EVENT_AUDIO_DEVICE_FORMAT_CHANGED = 0x1102

SDL_EVENT_SENSOR_UPDATE = 0x1200

SDL_EVENT_PEN_PROXIMITY_IN = 0x1300
SDL_EVENT_PEN_PROXIMITY_OUT = 0x1301
SDL_EVENT_PEN_DOWN = 0x1302
SDL_EVENT_PEN_UP = 0x1303
SDL_EVENT_PEN_BUTTON_DOWN = 0x1304
SDL_EVENT_PEN_BUTTON_UP = 0x1305
SDL_EVENT_PEN_MOTION = 0x1306
SDL_EVENT_PEN_AXIS = 0x1307

SDL_EVENT_CAMERA_DEVICE_ADDED = 0x1400
SDL_EVENT_CAMERA_DEVICE_REMOVED = 0x1401
SDL_EVENT_CAMERA_DEVICE_APPROVED = 0x1402
SDL_EVENT_CAMERA_DEVICE_DENIED = 0x1403

SDL_EVENT_RENDER_TARGETS_RESET = 0x2000
SDL_EVENT_RENDER_DEVICE_RESET = 0x2001
SDL_EVENT_RENDER_DEVICE_LOST = 0x2002

SDL_EVENT_PRIVATE0 = 0x4000
SDL_EVENT_PRIVATE1 = 0x4001
SDL_EVENT_PRIVATE2 = 0x4002
SDL_EVENT_PRIVATE3 = 0x4003

SDL_EVENT_POLL_SENTINEL = 0x7F00
SDL_EVENT_USER = 0x8000
SDL_EVENT_LAST = 0xFFFF
SDL_EVENT_ENUM_PADDING = 0x7FFFFFFF

class SDL_CommonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64)
    ]

class SDL_DisplayEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("displayID", SDL_DisplayID),
        ("data1", ctypes.c_int32),
        ("data2", ctypes.c_int32)
    ]

class SDL_WindowEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("data1", ctypes.c_int32),
        ("data2", ctypes.c_int32)
    ]

class SDL_KeyboardDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_KeyboardID)
    ]

class SDL_KeyboardEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_KeyboardID),
        ("scancode", SDL_Scancode),
        ("key", SDL_Keycode),
        ("mod", SDL_Keymod),
        ("raw", ctypes.c_uint16),
        ("down", ctypes.c_bool),
        ("repeat", ctypes.c_bool)
    ]

class SDL_TextEditingEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("text", ctypes.c_char_p),
        ("start", ctypes.c_int32),
        ("length", ctypes.c_int32)
    ]

class SDL_TextEditingCandidatesEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("candidates", ctypes.POINTER(ctypes.c_char_p)),
        ("num_candidates", ctypes.c_int32),
        ("selected_candidate", ctypes.c_int32),
        ("horizontal", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8)
    ]

class SDL_TextInputEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("text", ctypes.c_char_p)
    ]

class SDL_MouseDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_MouseID)
    ]

class SDL_MouseMotionEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_MouseID),
        ("state", SDL_MouseButtonFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("xrel", ctypes.c_float),
        ("yrel", ctypes.c_float)
    ]

class SDL_MouseButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_MouseID),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool),
        ("clicks", ctypes.c_uint8),
        ("padding", ctypes.c_uint8),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float)
    ]

class SDL_MouseWheelEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_MouseID),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("direction", SDL_MouseWheelDirection),
        ("mouse_x", ctypes.c_float),
        ("mouse_y", ctypes.c_float)
    ]

class SDL_JoyAxisEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("axis", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8),
        ("value", ctypes.c_int16),
        ("padding4", ctypes.c_uint16)
    ]

class SDL_JoyBallEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("ball", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8),
        ("xrel", ctypes.c_int16),
        ("yrel", ctypes.c_int16)
    ]

class SDL_JoyHatEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("hat", ctypes.c_uint8),
        ("value", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8)
    ]

class SDL_JoyButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8)
    ]

class SDL_JoyDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID)
    ]

class SDL_JoyBatteryEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("state", SDL_PowerState),
        ("percent", ctypes.c_int)
    ]

class SDL_GamepadAxisEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("axis", ctypes.c_uint8),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8),
        ("value", ctypes.c_int16),
        ("padding4", ctypes.c_uint16)
    ]

class SDL_GamepadButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8)
    ]

class SDL_GamepadDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID)
    ]

class SDL_GamepadTouchpadEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("touchpad", ctypes.c_int32),
        ("finger", ctypes.c_int32),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("pressure", ctypes.c_float)
    ]

class SDL_GamepadSensorEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_JoystickID),
        ("sensor", ctypes.c_int32),
        ("data", ctypes.c_float * 3),
        ("sensor_timestamp", ctypes.c_uint64)
    ]

class SDL_AudioDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_AudioDeviceID),
        ("recording", ctypes.c_bool),
        ("padding1", ctypes.c_uint8),
        ("padding2", ctypes.c_uint8),
        ("padding3", ctypes.c_uint8)
    ]

class SDL_CameraDeviceEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_CameraID)
    ]

class SDL_RenderEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID)
    ]

class SDL_TouchFingerEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("touchID", SDL_TouchID),
        ("fingerID", SDL_FingerID),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("dx", ctypes.c_float),
        ("dy", ctypes.c_float),
        ("pressure", ctypes.c_float),
        ("windowID", SDL_WindowID)
    ]

class SDL_PenProximityEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID)
    ]

class SDL_PenMotionEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float)
    ]

class SDL_PenTouchEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("eraser", ctypes.c_bool),
        ("down", ctypes.c_bool)
    ]

class SDL_PenButtonEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("button", ctypes.c_uint8),
        ("down", ctypes.c_bool)
    ]

class SDL_PenAxisEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("which", SDL_PenID),
        ("pen_state", SDL_PenInputFlags),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("axis", SDL_PenAxis),
        ("value", ctypes.c_float)
    ]

class SDL_DropEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("source", ctypes.c_char_p),
        ("data", ctypes.c_char_p)
    ]

class SDL_ClipboardEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("owner", ctypes.c_bool),
        ("num_mime_types", ctypes.c_int32),
        ("mime_types", ctypes.POINTER(ctypes.c_char_p))
    ]

class SDL_SensorEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("which", SDL_SensorID),
        ("data", ctypes.c_float * 6),
        ("sensor_timestamp", ctypes.c_uint64)
    ]

class SDL_QuitEvent(ctypes.Structure):
    _fields_ = [
        ("type", SDL_EventType),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64)
    ]

class SDL_UserEvent(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint32),
        ("reserved", ctypes.c_uint32),
        ("timestamp", ctypes.c_uint64),
        ("windowID", SDL_WindowID),
        ("code", ctypes.c_int32),
        ("data1", ctypes.c_void_p),
        ("data2", ctypes.c_void_p)
    ]

class SDL_Event(ctypes.Union):
    _fields_ = [
        ("type", ctypes.c_uint32),
        ("common", SDL_CommonEvent),
        ("display", SDL_DisplayEvent),
        ("window", SDL_WindowEvent),
        ("kdevice", SDL_KeyboardDeviceEvent),
        ("key", SDL_KeyboardEvent),
        ("edit", SDL_TextEditingEvent),
        ("edit_candidates", SDL_TextEditingCandidatesEvent),
        ("text", SDL_TextInputEvent),
        ("mdevice", SDL_MouseDeviceEvent),
        ("motion", SDL_MouseMotionEvent),
        ("button", SDL_MouseButtonEvent),
        ("wheel", SDL_MouseWheelEvent),
        ("jdevice", SDL_JoyDeviceEvent),
        ("jaxis", SDL_JoyAxisEvent),
        ("jball", SDL_JoyBallEvent),
        ("jhat", SDL_JoyHatEvent),
        ("jbutton", SDL_JoyButtonEvent),
        ("jbattery", SDL_JoyBatteryEvent),
        ("gdevice", SDL_GamepadDeviceEvent),
        ("gaxis", SDL_GamepadAxisEvent),
        ("gbutton", SDL_GamepadButtonEvent),
        ("gtouchpad", SDL_GamepadTouchpadEvent),
        ("gsensor", SDL_GamepadSensorEvent),
        ("adevice", SDL_AudioDeviceEvent),
        ("cdevice", SDL_CameraDeviceEvent),
        ("sensor", SDL_SensorEvent),
        ("quit", SDL_QuitEvent),
        ("user", SDL_UserEvent),
        ("tfinger", SDL_TouchFingerEvent),
        ("pproximity", SDL_PenProximityEvent),
        ("ptouch", SDL_PenTouchEvent),
        ("pmotion", SDL_PenMotionEvent),
        ("pbutton", SDL_PenButtonEvent),
        ("paxis", SDL_PenAxisEvent),
        ("render", SDL_RenderEvent),
        ("drop", SDL_DropEvent),
        ("clipboard", SDL_ClipboardEvent),
        ("padding", ctypes.c_uint8 * 128)
    ]

SDL_FUNC("SDL_PumpEvents", None)

SDL_EventAction = ctypes.c_int

SDL_ADDEVENT = 0
SDL_PEEKEVENT = 1
SDL_GETEVENT = 2

SDL_FUNC("SDL_PeepEvents", ctypes.c_int, ctypes.POINTER(SDL_Event), ctypes.c_int, SDL_EventAction, ctypes.c_uint32, ctypes.c_uint32)
SDL_FUNC("SDL_HasEvent", ctypes.c_bool, ctypes.c_uint32)
SDL_FUNC("SDL_HasEvents", ctypes.c_bool, ctypes.c_uint32, ctypes.c_uint32)
SDL_FUNC("SDL_FlushEvent", None, ctypes.c_uint32)
SDL_FUNC("SDL_FlushEvents", None, ctypes.c_uint32, ctypes.c_uint32)
SDL_FUNC("SDL_PollEvent", ctypes.c_bool, ctypes.POINTER(SDL_Event))
SDL_FUNC("SDL_WaitEvent", ctypes.c_bool, ctypes.POINTER(SDL_Event))
SDL_FUNC("SDL_WaitEventTimeout", ctypes.c_bool, ctypes.POINTER(SDL_Event), ctypes.c_int32)
SDL_FUNC("SDL_PushEvent", ctypes.c_bool, ctypes.POINTER(SDL_Event))

SDL_EventFilter = ctypes.CFUNCTYPE(ctypes.c_bool, ctypes.c_void_p, ctypes.POINTER(SDL_Event))

SDL_FUNC("SDL_SetEventFilter", None, SDL_EventFilter, ctypes.c_void_p)
SDL_FUNC("SDL_GetEventFilter", ctypes.c_bool, ctypes.POINTER(SDL_EventFilter), ctypes.POINTER(ctypes.c_void_p))
SDL_FUNC("SDL_AddEventWatch", ctypes.c_bool, SDL_EventFilter, ctypes.c_void_p)
SDL_FUNC("SDL_RemoveEventWatch", None, SDL_EventFilter, ctypes.c_void_p)
SDL_FUNC("SDL_FilterEvents", None, SDL_EventFilter, ctypes.c_void_p)
SDL_FUNC("SDL_SetEventEnabled", None, ctypes.c_uint32, ctypes.c_bool)
SDL_FUNC("SDL_EventEnabled", ctypes.c_bool, ctypes.c_uint32)
SDL_FUNC("SDL_RegisterEvents", ctypes.c_uint32, ctypes.c_int)
SDL_FUNC("SDL_GetWindowFromEvent", ctypes.POINTER(SDL_Window), ctypes.POINTER(SDL_Event))


from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_BINARY, SDL_BINARY

SDL_SET_CURRENT_BINARY(SDL_BINARY)

SDL_BlendMode = ctypes.c_uint32

SDL_BLENDMODE_NONE = 0x00000000
SDL_BLENDMODE_BLEND = 0x00000001
SDL_BLENDMODE_BLEND_PREMULTIPLIED = 0x00000010
SDL_BLENDMODE_ADD = 0x00000002
SDL_BLENDMODE_ADD_PREMULTIPLIED = 0x00000020
SDL_BLENDMODE_MOD = 0x00000004
SDL_BLENDMODE_MUL = 0x00000008
SDL_BLENDMODE_INVALID = 0x7FFFFFFF

SDL_BlendOperation = ctypes.c_int

SDL_BLENDOPERATION_ADD = 0x1
SDL_BLENDOPERATION_SUBTRACT = 0x2
SDL_BLENDOPERATION_REV_SUBTRACT = 0x3
SDL_BLENDOPERATION_MINIMUM = 0x4
SDL_BLENDOPERATION_MAXIMUM = 0x5

SDL_BlendFactor = ctypes.c_int

SDL_BLENDFACTOR_ZERO = 0x1
SDL_BLENDFACTOR_ONE = 0x2
SDL_BLENDFACTOR_SRC_COLOR = 0x3
SDL_BLENDFACTOR_ONE_MINUS_SRC_COLOR = 0x4
SDL_BLENDFACTOR_SRC_ALPHA = 0x5
SDL_BLENDFACTOR_ONE_MINUS_SRC_ALPHA = 0x6
SDL_BLENDFACTOR_DST_COLOR = 0x7
SDL_BLENDFACTOR_ONE_MINUS_DST_COLOR = 0x8
SDL_BLENDFACTOR_DST_ALPHA = 0x9
SDL_BLENDFACTOR_ONE_MINUS_DST_ALPHA = 0xA

SDL_FUNC("SDL_ComposeCustomBlendMode", SDL_BlendMode, SDL_BlendFactor, SDL_BlendFactor, SDL_BlendOperation, SDL_BlendFactor, SDL_BlendFactor, SDL_BlendOperation)


import os, ctypes, time

os.environ["SDL_MAIN_USE_CALLBACKS"] = "1"

import sdl3, colorsys

class AppState(ctypes.Structure):
    _fields_ = [
        ("window", sdl3.LP_SDL_Window),
        ("renderer", sdl3.LP_SDL_Renderer)
    ]

def SDL_AppInit(appstate: sdl3.LP_c_void_p, argc: ctypes.c_int, argv: sdl3.LP_c_char_p) -> sdl3.SDL_AppResult:
    appstate[0] = ctypes.cast(ctypes.pointer(state := AppState()), ctypes.c_void_p)

    if not sdl3.SDL_Init(sdl3.SDL_INIT_VIDEO):
        sdl3.SDL_Log("Couldn't initialize SDL: ".encode() + sdl3.SDL_GetError())
        return sdl3.SDL_APP_FAILURE

    if not sdl3.SDL_CreateWindowAndRenderer("Aermoss".encode(), 1600, 900, 0, ctypes.byref(state.window), ctypes.byref(state.renderer)):
        sdl3.SDL_Log("Couldn't create window/renderer: ".encode() + sdl3.SDL_GetError())
        return sdl3.SDL_APP_FAILURE

    return sdl3.SDL_APP_CONTINUE

def SDL_AppEvent(appstate: ctypes.c_void_p, event: sdl3.LP_SDL_Event) -> sdl3.SDL_AppResult:
    if sdl3.SDL_DEREFERENCE(event).type == sdl3.SDL_EVENT_QUIT:
        return sdl3.SDL_APP_SUCCESS
    
    return sdl3.SDL_APP_CONTINUE

def SDL_AppIterate(appstate: ctypes.c_void_p) -> sdl3.SDL_AppResult:
    state: AppState = sdl3.SDL_DEREFERENCE(ctypes.cast(appstate, ctypes.POINTER(AppState)))
    sdl3.SDL_SetRenderDrawColorFloat(state.renderer, *colorsys.hsv_to_rgb(time.time() / 3.0 % 1.0, 1.0, 0.1), sdl3.SDL_ALPHA_OPAQUE_FLOAT)
    sdl3.SDL_RenderClear(state.renderer)
    sdl3.SDL_RenderPresent(state.renderer)
    return sdl3.SDL_APP_CONTINUE

def SDL_AppQuit(appstate: ctypes.c_void_p, result: sdl3.SDL_AppResult) -> None:
    ...